@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,16 +10,21 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\Carbon;
 
 use Carbon\Carbon;
+use Generator;
+use InvalidArgumentException;
 use Tests\AbstractTestCase;
 
 class GettersTest extends AbstractTestCase
 {
     public function testGettersThrowExceptionOnUnknownGetter()
     {
-        $this->expectException(\InvalidArgumentException::class);
+        $this->expectExceptionObject(new InvalidArgumentException(
+            "Unknown getter 'doesNotExit'"
+        ));
 
         /** @var mixed $d */
         $d = Carbon::create(1234, 5, 6, 7, 8, 9);
@@ -191,6 +197,8 @@ class GettersTest extends AbstractTestCase
 
     public function testLocalizedGetters()
     {
+        Carbon::useStrictMode(false);
+
         $currentLocale = setlocale(LC_ALL, '0');
         if (setlocale(LC_ALL, 'fr_FR.UTF-8', 'fr_FR.utf8', 'fr_FR', 'fr') === false) {
             $this->markTestSkipped('testSetLocaleToAuto test need fr_FR.UTF-8.');
@@ -231,31 +239,29 @@ class GettersTest extends AbstractTestCase
     public function testGetAgeWithRealAge()
     {
         $d = Carbon::createFromDate(1975, 5, 21);
-        $age = intval(substr((string) (intval(date('Ymd')) - intval(date('Ymd', $d->timestamp))), 0, -4));
+        $age = (int) (substr((string) ((int) (date('Ymd')) - (int) (date('Ymd', $d->timestamp))), 0, -4));
 
         $this->assertSame($age, $d->age);
     }
 
-    public function dataProviderTestQuarter()
+    public static function dataForTestQuarter(): Generator
     {
-        return [
-            [1, 1],
-            [2, 1],
-            [3, 1],
-            [4, 2],
-            [5, 2],
-            [6, 2],
-            [7, 3],
-            [8, 3],
-            [9, 3],
-            [10, 4],
-            [11, 4],
-            [12, 4],
-        ];
+        yield [1, 1];
+        yield [2, 1];
+        yield [3, 1];
+        yield [4, 2];
+        yield [5, 2];
+        yield [6, 2];
+        yield [7, 3];
+        yield [8, 3];
+        yield [9, 3];
+        yield [10, 4];
+        yield [11, 4];
+        yield [12, 4];
     }
 
     /**
-     * @dataProvider \Tests\Carbon\GettersTest::dataProviderTestQuarter
+     * @dataProvider \Tests\Carbon\GettersTest::dataForTestQuarter
      *
      * @param int $month
      * @param int $quarter
@@ -267,7 +273,7 @@ class GettersTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\GettersTest::dataProviderTestQuarter
+     * @dataProvider \Tests\Carbon\GettersTest::dataForTestQuarter
      *
      * @param int $month
      * @param int $quarter
@@ -279,7 +285,7 @@ class GettersTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\GettersTest::dataProviderTestQuarter
+     * @dataProvider \Tests\Carbon\GettersTest::dataForTestQuarter
      *
      * @param int $month
      * @param int $quarter
@@ -293,7 +299,7 @@ class GettersTest extends AbstractTestCase
     public function testGetLocalTrue()
     {
         // Default timezone has been set to America/Toronto in AbstractTestCase.php
-        // @see : http://en.wikipedia.org/wiki/List_of_UTC_time_offsets
+        // @see : https://en.wikipedia.org/wiki/List_of_UTC_time_offsets
         $this->assertTrue(Carbon::createFromDate(2012, 1, 1, 'America/Toronto')->local);
         $this->assertTrue(Carbon::createFromDate(2012, 1, 1, 'America/Toronto')->isLocal());
         $this->assertTrue(Carbon::createFromDate(2012, 1, 1, 'America/New_York')->local);
