@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\Carbon;
 
 use Carbon\Carbon;
@@ -16,7 +18,8 @@ use Carbon\CarbonInterval;
 use Carbon\Exceptions\NotLocaleAwareException;
 use Carbon\Language;
 use Carbon\Translator;
-use ReflectionMethod;
+use Generator;
+use InvalidArgumentException;
 use Symfony\Component\Translation\IdentityTranslator;
 use Symfony\Component\Translation\Loader\ArrayLoader;
 use Symfony\Component\Translation\MessageCatalogue;
@@ -25,6 +28,9 @@ use Symfony\Component\Translation\TranslatorInterface;
 use Tests\AbstractTestCase;
 use Tests\Carbon\Fixtures\MyCarbon;
 
+/**
+ * @group localization
+ */
 class LocalizationTest extends AbstractTestCase
 {
     protected function tearDown(): void
@@ -73,7 +79,7 @@ class LocalizationTest extends AbstractTestCase
         setlocale(LC_ALL, $currentLocale);
         rename(__DIR__.'/../../src/Carbon/Lang/disabled_ar_AE.php', __DIR__.'/../../src/Carbon/Lang/ar_AE.php');
 
-        $this->assertSame('ar', $locale);
+        $this->assertStringStartsWith('ar', $locale);
         $this->assertSame('منذ ثانيتين', $diff);
 
         if (setlocale(LC_ALL, 'sr_ME.UTF-8', 'sr_ME.utf8', 'sr_ME', 'sr') === false) {
@@ -166,152 +172,150 @@ class LocalizationTest extends AbstractTestCase
      * @see \Tests\Carbon\LocalizationTest::testSetLocale
      * @see \Tests\Carbon\LocalizationTest::testSetTranslator
      *
-     * @return array
+     * @return \Generator
      */
-    public function providerLocales()
+    public static function dataForLocales(): Generator
     {
-        return [
-            ['af'],
-            ['ar'],
-            ['ar_DZ'],
-            ['ar_KW'],
-            ['ar_LY'],
-            ['ar_MA'],
-            ['ar_SA'],
-            ['ar_Shakl'],
-            ['ar_TN'],
-            ['az'],
-            ['be'],
-            ['bg'],
-            ['bm'],
-            ['bn'],
-            ['bo'],
-            ['br'],
-            ['bs'],
-            ['bs_BA'],
-            ['ca'],
-            ['cs'],
-            ['cv'],
-            ['cy'],
-            ['da'],
-            ['de'],
-            ['de_AT'],
-            ['de_CH'],
-            ['dv'],
-            ['dv_MV'],
-            ['el'],
-            ['en'],
-            ['en_AU'],
-            ['en_CA'],
-            ['en_GB'],
-            ['en_IE'],
-            ['en_IL'],
-            ['en_NZ'],
-            ['eo'],
-            ['es'],
-            ['es_DO'],
-            ['es_US'],
-            ['et'],
-            ['eu'],
-            ['fa'],
-            ['fi'],
-            ['fo'],
-            ['fr'],
-            ['fr_CA'],
-            ['fr_CH'],
-            ['fy'],
-            ['gd'],
-            ['gl'],
-            ['gom_Latn'],
-            ['gu'],
-            ['he'],
-            ['hi'],
-            ['hr'],
-            ['hu'],
-            ['hy'],
-            ['hy_AM'],
-            ['id'],
-            ['is'],
-            ['it'],
-            ['ja'],
-            ['jv'],
-            ['ka'],
-            ['kk'],
-            ['km'],
-            ['kn'],
-            ['ko'],
-            ['ku'],
-            ['ky'],
-            ['lb'],
-            ['lo'],
-            ['lt'],
-            ['lv'],
-            ['me'],
-            ['mi'],
-            ['mk'],
-            ['ml'],
-            ['mn'],
-            ['mr'],
-            ['ms'],
-            ['ms_MY'],
-            ['mt'],
-            ['my'],
-            ['nb'],
-            ['ne'],
-            ['nl'],
-            ['nl_BE'],
-            ['nn'],
-            ['no'],
-            ['oc'],
-            ['pa_IN'],
-            ['pl'],
-            ['ps'],
-            ['pt'],
-            ['pt_BR'],
-            ['ro'],
-            ['ru'],
-            ['sd'],
-            ['se'],
-            ['sh'],
-            ['si'],
-            ['sk'],
-            ['sl'],
-            ['sq'],
-            ['sr'],
-            ['sr_Cyrl'],
-            ['sr_Cyrl_ME'],
-            ['sr_Latn_ME'],
-            ['sr_ME'],
-            ['ss'],
-            ['sv'],
-            ['sw'],
-            ['ta'],
-            ['te'],
-            ['tet'],
-            ['tg'],
-            ['th'],
-            ['tl_PH'],
-            ['tlh'],
-            ['tr'],
-            ['tzl'],
-            ['tzm'],
-            ['tzm_Latn'],
-            ['ug_CN'],
-            ['uk'],
-            ['ur'],
-            ['uz'],
-            ['uz_Latn'],
-            ['vi'],
-            ['yo'],
-            ['zh'],
-            ['zh_CN'],
-            ['zh_HK'],
-            ['zh_TW'],
-        ];
+        yield ['af'];
+        yield ['ar'];
+        yield ['ar_DZ'];
+        yield ['ar_KW'];
+        yield ['ar_LY'];
+        yield ['ar_MA'];
+        yield ['ar_SA'];
+        yield ['ar_Shakl'];
+        yield ['ar_TN'];
+        yield ['az'];
+        yield ['be'];
+        yield ['bg'];
+        yield ['bm'];
+        yield ['bn'];
+        yield ['bo'];
+        yield ['br'];
+        yield ['bs'];
+        yield ['bs_BA'];
+        yield ['ca'];
+        yield ['cs'];
+        yield ['cv'];
+        yield ['cy'];
+        yield ['da'];
+        yield ['de'];
+        yield ['de_AT'];
+        yield ['de_CH'];
+        yield ['dv'];
+        yield ['dv_MV'];
+        yield ['el'];
+        yield ['en'];
+        yield ['en_AU'];
+        yield ['en_CA'];
+        yield ['en_GB'];
+        yield ['en_IE'];
+        yield ['en_IL'];
+        yield ['en_NZ'];
+        yield ['eo'];
+        yield ['es'];
+        yield ['es_DO'];
+        yield ['es_US'];
+        yield ['et'];
+        yield ['eu'];
+        yield ['fa'];
+        yield ['fi'];
+        yield ['fo'];
+        yield ['fr'];
+        yield ['fr_CA'];
+        yield ['fr_CH'];
+        yield ['fy'];
+        yield ['gd'];
+        yield ['gl'];
+        yield ['gom_Latn'];
+        yield ['gu'];
+        yield ['he'];
+        yield ['hi'];
+        yield ['hr'];
+        yield ['hu'];
+        yield ['hy'];
+        yield ['hy_AM'];
+        yield ['id'];
+        yield ['is'];
+        yield ['it'];
+        yield ['ja'];
+        yield ['jv'];
+        yield ['ka'];
+        yield ['kk'];
+        yield ['km'];
+        yield ['kn'];
+        yield ['ko'];
+        yield ['ku'];
+        yield ['ky'];
+        yield ['lb'];
+        yield ['lo'];
+        yield ['lt'];
+        yield ['lv'];
+        yield ['me'];
+        yield ['mi'];
+        yield ['mk'];
+        yield ['ml'];
+        yield ['mn'];
+        yield ['mr'];
+        yield ['ms'];
+        yield ['ms_MY'];
+        yield ['mt'];
+        yield ['my'];
+        yield ['nb'];
+        yield ['ne'];
+        yield ['nl'];
+        yield ['nl_BE'];
+        yield ['nn'];
+        yield ['no'];
+        yield ['oc'];
+        yield ['pa_IN'];
+        yield ['pl'];
+        yield ['ps'];
+        yield ['pt'];
+        yield ['pt_BR'];
+        yield ['ro'];
+        yield ['ru'];
+        yield ['sd'];
+        yield ['se'];
+        yield ['sh'];
+        yield ['si'];
+        yield ['sk'];
+        yield ['sl'];
+        yield ['sq'];
+        yield ['sr'];
+        yield ['sr_Cyrl'];
+        yield ['sr_Cyrl_ME'];
+        yield ['sr_Latn_ME'];
+        yield ['sr_ME'];
+        yield ['ss'];
+        yield ['sv'];
+        yield ['sw'];
+        yield ['ta'];
+        yield ['te'];
+        yield ['tet'];
+        yield ['tg'];
+        yield ['th'];
+        yield ['tl_PH'];
+        yield ['tlh'];
+        yield ['tr'];
+        yield ['tzl'];
+        yield ['tzm'];
+        yield ['tzm_Latn'];
+        yield ['ug_CN'];
+        yield ['uk'];
+        yield ['ur'];
+        yield ['uz'];
+        yield ['uz_Latn'];
+        yield ['vi'];
+        yield ['yo'];
+        yield ['zh'];
+        yield ['zh_CN'];
+        yield ['zh_HK'];
+        yield ['zh_TW'];
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::providerLocales
+     * @dataProvider \Tests\Carbon\LocalizationTest::dataForLocales
      *
      * @param string $locale
      */
@@ -322,7 +326,7 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::providerLocales
+     * @dataProvider \Tests\Carbon\LocalizationTest::dataForLocales
      *
      * @param string $locale
      */
@@ -349,24 +353,22 @@ class LocalizationTest extends AbstractTestCase
     /**
      * @see \Tests\Carbon\LocalizationTest::testSetLocaleWithMalformedLocale
      *
-     * @return array
+     * @return \Generator
      */
-    public function dataProviderTestSetLocaleWithMalformedLocale()
+    public static function dataForTestSetLocaleWithMalformedLocale(): Generator
     {
-        return [
-            ['DE'],
-            ['pt-BR'],
-            ['pt-br'],
-            ['PT-br'],
-            ['PT-BR'],
-            ['pt_br'],
-            ['PT_br'],
-            ['PT_BR'],
-        ];
+        yield ['DE'];
+        yield ['pt-BR'];
+        yield ['pt-br'];
+        yield ['PT-br'];
+        yield ['PT-BR'];
+        yield ['pt_br'];
+        yield ['PT_br'];
+        yield ['PT_BR'];
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::dataProviderTestSetLocaleWithMalformedLocale
+     * @dataProvider \Tests\Carbon\LocalizationTest::dataForTestSetLocaleWithMalformedLocale
      *
      * @param string $malformedLocale
      */
@@ -682,7 +684,7 @@ class LocalizationTest extends AbstractTestCase
 
     public function testGetAvailableLocales()
     {
-        $this->assertCount(count(glob(__DIR__.'/../../src/Carbon/Lang/*.php')), Carbon::getAvailableLocales());
+        $this->assertCount(\count(glob(__DIR__.'/../../src/Carbon/Lang/*.php')), Carbon::getAvailableLocales());
 
         /** @var Translator $translator */
         $translator = Carbon::getTranslator();
@@ -696,30 +698,21 @@ class LocalizationTest extends AbstractTestCase
     public function testNotLocaleAwareException()
     {
         if (method_exists(TranslatorInterface::class, 'getLocale')) {
-            $this->markTestSkipped('In Symfony < 4, NotLocaleAwareException will never been thrown.');
+            $this->markTestSkipped('In Symfony < 5, NotLocaleAwareException will never been thrown.');
         }
 
-        $parameters = (new ReflectionMethod(TranslatorInterface::class, 'trans'))->getParameters();
-
-        /** @var TranslatorInterface $translator */
-        $translator = $parameters[0]->getType()->getName() === 'string'
-            ? (new class implements TranslatorInterface {
-                public function trans(string $id, array $parameters = [], string $domain = null, string $locale = null)
-                {
-                    return 'x';
-                }
-            })
-            : (new class implements TranslatorInterface {
-                public function trans($id, array $parameters = [], $domain = null, $locale = null)
-                {
-                    return 'x';
-                }
-            });
+        $translator = new class() implements TranslatorInterface {
+            public function trans(string $id, array $parameters = [], ?string $domain = null, ?string $locale = null)
+            {
+                return 'x';
+            }
+        };
 
         Carbon::setTranslator($translator);
 
-        $this->expectException(NotLocaleAwareException::class);
-        $this->expectExceptionMessage(get_class($translator).' does neither implements Symfony\\Contracts\\Translation\\LocaleAwareInterface nor getLocale() method.');
+        $this->expectExceptionObject(new NotLocaleAwareException(
+            $translator
+        ));
 
         Carbon::now()->locale();
     }
@@ -727,7 +720,7 @@ class LocalizationTest extends AbstractTestCase
     public function testGetAvailableLocalesInfo()
     {
         $infos = Carbon::getAvailableLocalesInfo();
-        $this->assertCount(count(Carbon::getAvailableLocales()), Carbon::getAvailableLocalesInfo());
+        $this->assertCount(\count(Carbon::getAvailableLocales()), Carbon::getAvailableLocalesInfo());
         $this->assertArrayHasKey('en', $infos);
         $this->assertInstanceOf(Language::class, $infos['en']);
         $this->assertSame('English', $infos['en']->getIsoName());
@@ -740,6 +733,13 @@ class LocalizationTest extends AbstractTestCase
         $this->assertSame('3 კვირაში', $diff);
     }
 
+    public function testSinhaleseSpecialAfterTranslation()
+    {
+        $diff = Carbon::now()->locale('si')->addDays(3)->diffForHumans(Carbon::now());
+
+        $this->assertSame('දින 3 න්', $diff);
+    }
+
     public function testWeekDayMultipleForms()
     {
         $date = Carbon::parse('2018-10-10')->locale('ru');
@@ -758,16 +758,17 @@ class LocalizationTest extends AbstractTestCase
 
     public function testTranslationCustomWithCustomTranslator()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Translator does not implement Symfony\Component\Translation\TranslatorInterface '.
-            'and Symfony\Component\Translation\TranslatorBagInterface.'
-        );
+            'and Symfony\Component\Translation\TranslatorBagInterface. '.
+            'Symfony\Component\Translation\IdentityTranslator has been given.'
+        ));
 
         $date = Carbon::create(2018, 1, 1, 0, 0, 0);
-        $date->setLocalTranslator(class_exists(MessageSelector::class)
-            ? new IdentityTranslator(new MessageSelector())
-            : new IdentityTranslator()
+        $date->setLocalTranslator(
+            class_exists(MessageSelector::class)
+                ? new IdentityTranslator(new MessageSelector())
+                : new IdentityTranslator()
         );
 
         $date->getTranslationMessage('foo');
@@ -840,7 +841,7 @@ class LocalizationTest extends AbstractTestCase
             'join' => true,
         ]));
 
-        Carbon::setTranslator(new \Symfony\Component\Translation\IdentityTranslator());
+        Carbon::setTranslator(new IdentityTranslator());
 
         $this->assertNull(Carbon::getFallbackLocale());
 
@@ -913,4 +914,28 @@ class LocalizationTest extends AbstractTestCase
             Carbon::parse('2020-02-15')->locale('de')->monthName
         );
     }
+
+    public function testDeclensionModes()
+    {
+        $this->assertSame(
+            '2 жил 3 сар 1 өдөр 1с өмнө',
+            Carbon::now()
+                ->subYears(2)
+                ->subMonths(3)
+                ->subDay()
+                ->subSecond()
+                ->locale('mn')
+                ->diffForHumans(null, null, true, 4)
+        );
+        $this->assertSame(
+            '2 жил 3 сар 1 өдөр 1 секундын өмнө',
+            Carbon::now()
+                ->subYears(2)
+                ->subMonths(3)
+                ->subDay()
+                ->subSecond()
+                ->locale('mn')
+                ->diffForHumans(null, null, false, 4)
+        );
+    }
 }
