@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\Carbon;
 
 use Carbon\Carbon;
@@ -17,7 +19,7 @@ use Tests\AbstractTestCase;
 
 class WeekTest extends AbstractTestCase
 {
-    const SAMPLE = [
+    public const SAMPLE = [
         '1990-12-20' => [
             1990,
             1990,
@@ -5428,5 +5430,11 @@ class WeekTest extends AbstractTestCase
         $this->assertSame('08-06', $date->isoWeekday(CarbonInterface::MONDAY)->format('m-d'));
         $this->assertSame(CarbonInterface::MONDAY, $date->weekday());
         $this->assertSame(CarbonInterface::MONDAY, $date->isoWeekday());
+
+        Carbon::setLocale('nb');
+        Carbon::setTestNowAndTimezone(Carbon::parse('2022-09-19 11:00:00'), 'Europe/Oslo');
+        $this->assertSame('2022-09-20 00:00', Carbon::today()->weekday(CarbonInterface::TUESDAY)->format('Y-m-d H:i'));
+        Carbon::setTestNowAndTimezone(Carbon::parse('2022-10-21 16:00:00'), 'Europe/Oslo');
+        $this->assertSame('2022-10-18 00:00', Carbon::today()->weekday(CarbonInterface::TUESDAY)->format('Y-m-d H:i'));
     }
 }
