@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonImmutable;
 
 use Carbon\CarbonImmutable as Carbon;
@@ -16,7 +18,7 @@ use Carbon\CarbonInterval;
 use Carbon\Exceptions\NotLocaleAwareException;
 use Carbon\Language;
 use Carbon\Translator;
-use ReflectionMethod;
+use InvalidArgumentException;
 use Symfony\Component\Translation\IdentityTranslator;
 use Symfony\Component\Translation\Loader\ArrayLoader;
 use Symfony\Component\Translation\MessageCatalogue;
@@ -25,6 +27,9 @@ use Symfony\Component\Translation\TranslatorInterface;
 use Tests\AbstractTestCase;
 use Tests\CarbonImmutable\Fixtures\MyCarbon;
 
+/**
+ * @group localization
+ */
 class LocalizationTest extends AbstractTestCase
 {
     protected function tearDown(): void
@@ -73,7 +78,7 @@ class LocalizationTest extends AbstractTestCase
         setlocale(LC_ALL, $currentLocale);
         rename(__DIR__.'/../../src/Carbon/Lang/disabled_ar_AE.php', __DIR__.'/../../src/Carbon/Lang/ar_AE.php');
 
-        $this->assertSame('ar', $locale);
+        $this->assertStringStartsWith('ar', $locale);
         $this->assertSame('منذ ثانيتين', $diff);
 
         if (setlocale(LC_ALL, 'sr_ME.UTF-8', 'sr_ME.utf8', 'sr_ME', 'sr') === false) {
@@ -163,12 +168,12 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @see \Tests\Carbon\LocalizationTest::testSetLocale
-     * @see \Tests\Carbon\LocalizationTest::testSetTranslator
+     * @see \Tests\CarbonImmutable\LocalizationTest::testSetLocale
+     * @see \Tests\CarbonImmutable\LocalizationTest::testSetTranslator
      *
      * @return array
      */
-    public function providerLocales()
+    public static function dataForLocales()
     {
         return [
             ['af'],
@@ -311,7 +316,7 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::providerLocales
+     * @dataProvider \Tests\CarbonImmutable\LocalizationTest::dataForLocales
      *
      * @param string $locale
      */
@@ -322,7 +327,7 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::providerLocales
+     * @dataProvider \Tests\CarbonImmutable\LocalizationTest::dataForLocales
      *
      * @param string $locale
      */
@@ -346,11 +351,11 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @see \Tests\Carbon\LocalizationTest::testSetLocaleWithMalformedLocale
+     * @see \Tests\CarbonImmutable\LocalizationTest::testSetLocaleWithMalformedLocale
      *
      * @return array
      */
-    public function dataProviderTestSetLocaleWithMalformedLocale()
+    public static function dataForTestSetLocaleWithMalformedLocale()
     {
         return [
             ['DE'],
@@ -365,7 +370,7 @@ class LocalizationTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\LocalizationTest::dataProviderTestSetLocaleWithMalformedLocale
+     * @dataProvider \Tests\CarbonImmutable\LocalizationTest::dataForTestSetLocaleWithMalformedLocale
      *
      * @param string $malformedLocale
      */
@@ -651,7 +656,7 @@ class LocalizationTest extends AbstractTestCase
 
     public function testGetAvailableLocales()
     {
-        $this->assertCount(count(glob(__DIR__.'/../../src/Carbon/Lang/*.php')), Carbon::getAvailableLocales());
+        $this->assertCount(\count(glob(__DIR__.'/../../src/Carbon/Lang/*.php')), Carbon::getAvailableLocales());
 
         /** @var Translator $translator */
         $translator = Carbon::getTranslator();
@@ -665,30 +670,19 @@ class LocalizationTest extends AbstractTestCase
     public function testNotLocaleAwareException()
     {
         if (method_exists(TranslatorInterface::class, 'getLocale')) {
-            $this->markTestSkipped('In Symfony < 4, NotLocaleAwareException will never been thrown.');
+            $this->markTestSkipped('In Symfony < 5, NotLocaleAwareException will never been thrown.');
         }
 
-        $parameters = (new ReflectionMethod(TranslatorInterface::class, 'trans'))->getParameters();
-
-        /** @var TranslatorInterface $translator */
-        $translator = $parameters[0]->getType()->getName() === 'string'
-            ? (new class implements TranslatorInterface {
-                public function trans(string $id, array $parameters = [], string $domain = null, string $locale = null)
-                {
-                    return 'x';
-                }
-            })
-            : (new class implements TranslatorInterface {
-                public function trans($id, array $parameters = [], $domain = null, $locale = null)
-                {
-                    return 'x';
-                }
-            });
+        $translator = new class() implements TranslatorInterface {
+            public function trans(string $id, array $parameters = [], ?string $domain = null, ?string $locale = null)
+            {
+                return 'x';
+            }
+        };
 
         Carbon::setTranslator($translator);
 
-        $this->expectException(NotLocaleAwareException::class);
-        $this->expectExceptionMessage(get_class($translator).' does neither implements Symfony\\Contracts\\Translation\\LocaleAwareInterface nor getLocale() method.');
+        $this->expectExceptionObject(new NotLocaleAwareException($translator));
 
         Carbon::now()->locale();
     }
@@ -696,7 +690,7 @@ class LocalizationTest extends AbstractTestCase
     public function testGetAvailableLocalesInfo()
     {
         $infos = Carbon::getAvailableLocalesInfo();
-        $this->assertCount(count(Carbon::getAvailableLocales()), Carbon::getAvailableLocalesInfo());
+        $this->assertCount(\count(Carbon::getAvailableLocales()), Carbon::getAvailableLocalesInfo());
         $this->assertArrayHasKey('en', $infos);
         $this->assertInstanceOf(Language::class, $infos['en']);
         $this->assertSame('English', $infos['en']->getIsoName());
@@ -727,16 +721,17 @@ class LocalizationTest extends AbstractTestCase
 
     public function testTranslationCustomWithCustomTranslator()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Translator does not implement Symfony\Component\Translation\TranslatorInterface '.
-            'and Symfony\Component\Translation\TranslatorBagInterface.'
-        );
+            'and Symfony\Component\Translation\TranslatorBagInterface. '.
+            'Symfony\Component\Translation\IdentityTranslator has been given.'
+        ));
 
         $date = Carbon::create(2018, 1, 1, 0, 0, 0);
-        $date->setLocalTranslator(class_exists(MessageSelector::class)
-            ? new IdentityTranslator(new MessageSelector())
-            : new IdentityTranslator()
+        $date->setLocalTranslator(
+            class_exists(MessageSelector::class)
+                ? new IdentityTranslator(new MessageSelector())
+                : new IdentityTranslator()
         );
 
         $date->getTranslationMessage('foo');
@@ -806,7 +801,7 @@ class LocalizationTest extends AbstractTestCase
             'join' => true,
         ]));
 
-        Carbon::setTranslator(new \Symfony\Component\Translation\IdentityTranslator());
+        Carbon::setTranslator(new IdentityTranslator());
 
         $this->assertNull(Carbon::getFallbackLocale());
 
@@ -856,4 +851,41 @@ class LocalizationTest extends AbstractTestCase
             Carbon::now()->subHour()->locale('uk')->diffForHumans(['aUnit' => true])
         );
     }
+
+    public function testPolishDeclensions()
+    {
+        $hour = Carbon::now()->addHour()->locale('pl');
+        $minute = Carbon::now()->addMinute()->locale('pl');
+        $second = Carbon::now()->addSecond()->locale('pl');
+        $this->assertSame('za 1 godzinę', $hour->diffForHumans());
+        $this->assertSame('za 1 minutę', $minute->diffForHumans());
+        $this->assertSame('za 1 sekundę', $second->diffForHumans());
+        $this->assertSame('za godzinę', $hour->diffForHumans(['aUnit' => true]));
+        $this->assertSame('za minutę', $minute->diffForHumans(['aUnit' => true]));
+        $this->assertSame('za sekundę', $second->translate('from_now', ['time' => 'sekunda']));
+    }
+
+    public function testDeclensionModes()
+    {
+        $this->assertSame(
+            '2 жил 3 сар 1 өдөр 1с өмнө',
+            Carbon::now()
+                ->subYears(2)
+                ->subMonths(3)
+                ->subDay()
+                ->subSecond()
+                ->locale('mn')
+                ->diffForHumans(null, null, true, 4)
+        );
+        $this->assertSame(
+            '2 жил 3 сар 1 өдөр 1 секундын өмнө',
+            Carbon::now()
+                ->subYears(2)
+                ->subMonths(3)
+                ->subDay()
+                ->subSecond()
+                ->locale('mn')
+                ->diffForHumans(null, null, false, 4)
+        );
+    }
 }
