@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonImmutable;
 
 use Carbon\CarbonImmutable as Carbon;
@@ -17,7 +19,7 @@ use Tests\AbstractTestCase;
 
 class WeekTest extends AbstractTestCase
 {
-    const SAMPLE = [
+    public const SAMPLE = [
         '1990-12-20' => [
             1990,
             1990,
